@@ -3,7 +3,7 @@ import typing
 import random
 import string
 from functools import reduce
-from typing import TypeVar, Callable, Generic, Iterable, Any, Union, Tuple
+from typing import TypeVar, Callable, Generic, Iterable, Any, Union, Tuple, Type
 
 from toolz.itertoolz import cons, groupby
 
@@ -165,7 +165,7 @@ class List(Generic[A], typing.List[A], Implicits, implicits=True, metaclass=List
         l, r = reduce(splitter, self, ((), (),))  # type: ignore
         return List.wrap(l), List.wrap(r)
 
-    def split_type(self, tpe: type) -> Tuple['List[A]', 'List[A]']:
+    def split_type(self, tpe: Type[B]) -> Tuple['List[B]', 'List[A]']:
         return self.split(lambda a: isinstance(a, tpe))
 
     def index_of(self, target: Any) -> maybe.Maybe[int]:
@@ -173,7 +173,7 @@ class List(Generic[A], typing.List[A], Implicits, implicits=True, metaclass=List
 
     @property
     def reversed(self) -> 'List[A]':
-        return Lists.wrap(reversed(self))  # type: ignore
+        return Lists.wrap(reversed(self))
 
     def mk_string(self, sep: str='') -> str:
         return sep.join(self / str)
@@ -278,8 +278,12 @@ class List(Generic[A], typing.List[A], Implicits, implicits=True, metaclass=List
     def rstrip_newlines(self) -> 'List[str]':
         return self / (lambda a: a.rstrip('\n'))
 
+    def collect(self, f: Callable[[A], maybe.Maybe[B]]) -> 'List[B]':
+        return self.flat_map(f)
+
 
 class Lists:
+    empty = List()
 
     @staticmethod
     def wrap(l: Iterable[B]) -> List[B]:
@@ -325,4 +329,6 @@ class Lists:
     def iff_l(cond: bool, a: Union[A, Callable[[], A]]) -> List[A]:
         return call_by_name(a) if cond else List()
 
-__all__ = ('List',)
+Nil = Lists.empty
+
+__all__ = ('List', 'Lists', 'Nil')
